
#include <iostream>
#include <GL/glut.h>
#include <fstream>
#include "world.h"
#include <math.h>


using namespace std;

// I don't know why this isn't already defined from one of the above headers, but it is now
#define NULL 0

const float World::BALL_RADIUS = 0.05f; // units: GL units
const float World::ELASTICITY = 0.60f;  // scaling factor applied to new velocity's perpendicular component after collision
const float World::FRICTION = 0.0005f;  // friction on each collision
const float World::GRAVITY = 0.005f;    // units: (GL units) / timestep^2
const Vector World::HOLE_POSITION = Vector(-7.63, -0.66, -9.48); // position of the hole (hardcoded)
const float World::HOLE_RADIUS = 0.1f;
const Vector World::TEE_POSITION = Vector(0.5, 0.55, 1.3);    // Hollywood level
//const Vector World::TEE_POSITION = Vector(-.3, .1, 0);      // edge test level (input3.txt)

World* World::pinstance = NULL;
World* World::instance() 
{
	if(pinstance == NULL)
      pinstance = new World; // create the singleton World instance if not yet created

	return pinstance;
}

World::World(void)
: theBall(TEE_POSITION, BALL_RADIUS, Color(1.0, 1.0, 1.0)), theCamera(),  numTriangles(0), triangles(NULL), theClub(),
	mode(AIM), numSlowSteps(0), power(0), theHole(HOLE_POSITION, HOLE_RADIUS), numStrokes(0)
{
	theBall.setVelocity(Vector(0.0, -0.01, 0.01));
	theCamera.setPosition(5, 0, 25);
	theCamera.lookAt(theBall.getPosition());
	theClub.aimAt(theBall.getPosition());
	// register glut event handlers
	glutMotionFunc(mouseMoveCallback);    
	glutMouseFunc(mousePressCallback);
	glutKeyboardFunc(keyPressCallback);
}

// Read world triangles from file
bool World::loadFile(char* fileName)
{
	cout << "Reading file " << fileName << endl;
	ifstream inFile(fileName, ios::in);  // open the input file

	if(!inFile.good())
	{
		cout << "Couldn't open " << fileName << endl;
		return false;                    // the filename wasn't valid
	}
	// New input parsing code, following Z's format
	int numVertices;
	inFile >> numVertices;

	Vector* vertices = new Vector[numVertices];

	// Load in the list of vertices
	for(int i = 0; i < numVertices; i++)
		inFile >> vertices[i].x >> vertices[i].y >> vertices[i].z;

	inFile >> numTriangles;
	triangles = new Triangle*[numTriangles];

	// Load in the list of triangles with colors
	for(int i = 0; i < numTriangles; i++)
	{
		int v1, v2, v3;
		float r, g, b;
		inFile >> v1 >> v2 >> v3 >> r >> g >> b;
		triangles[i] = new Triangle(vertices[v1], vertices[v2], vertices[v3], Color(r,g,b));
	}

	delete[] vertices;

	return true;  // Successfully loaded the file
}

World::~World(void)
{
	glutMotionFunc(NULL);     // unregister glut event handlers
	glutMouseFunc(NULL);
	glutKeyboardFunc(NULL);
	for(int x = 0; x < numTriangles; ++x)
		delete triangles[x];
	delete[] triangles;
}

void World::draw()
{
	// Initialize GL for a new drawing run
	glLoadIdentity();
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glDisable(GL_TEXTURE_2D);
	glPolygonMode(GL_FRONT, GL_FILL);
	glPolygonMode(GL_BACK, GL_LINE);

	// First we apply the camera transform to the current matrix.  Then we draw the triangles and hole.
	// If the ball isn't in the hole yet, draw it.  Also draw the club if we're not running the physics sim.
	theCamera.doTransform();
	for(int x = 0; x < numTriangles; ++x)
		triangles[x]->draw();
	theHole.draw();
	if(mode != DONE)
	{
		theBall.draw();
		glPolygonMode(GL_BACK, GL_FILL);
		if(mode != SIM)
			theClub.draw();
	}
	glutSwapBuffers();
}

bool World::advanceTimestep()
{
	// if game is already over, just return
	if(mode == DONE)
		return false;

	// check if the ball is in the hole
	if(theHole.isIn(theBall.getPosition() - Vector(0.0, theBall.getRadius(), 0.0)))
	{
		mode = DONE;
		char message[100];
		sprintf(message, "You win!\nYou took %i strokes.", numStrokes);
		MessageBox(NULL, message, "Congratulations", MB_OK);
		return false;
	}

	// apply some gravity
	theBall.setVelocity(theBall.getVelocity() - Vector(0.0, GRAVITY, 0.0));

	return advanceTime(1.0f);
}

bool World::advanceTime(float timeStep)
{
	Vector startPos = theBall.getPosition();
	Vector endPos = startPos + (theBall.getVelocity() * timeStep);

	theCamera.lookAt(endPos);

	Vector pathBoundingSphereCenter = (startPos + endPos) * 0.5;
	float pathBoundingSphereRadius = sqrt((endPos - pathBoundingSphereCenter).magSq())
									 + theBall.getRadius();

	// check for collisions with the triangles
	CollisionType collisionFound = NONE;
	Collision* faceCollision = NULL;
	Vector edgeOrCornerCollisionPt; // point ON THE TRIANGLE of the collision; you can calculate
	                                // point on the ball's PATH of the collision from previousAlpha
//	float previousAlpha = 1.0f; // alpha value from an edge or corner collision, representing
//	                            // distance along path (0.0 to 1.0) the collision was at
	float smallestAlpha = 1.01f;

	for(int x = 0; x < numTriangles; ++x)
	{
		// only do full collision check if bounding spheres intersect
		if(boundingSphereIntersection(pathBoundingSphereCenter, pathBoundingSphereRadius,
									  triangles[x]->getBoundingSphereCenter(),
									  triangles[x]->getBoundingSphereRadius()))
		{
			// Check if it collides with a face
			Collision* newFaceCollision = checkFaceCollision(startPos, endPos,
											                 theBall.getRadius(), triangles[x]);
			if(newFaceCollision != NULL) // there was a face collision
			{
				if(faceCollision != NULL)
				{
					// if new collision is closer than old one, keep it and throw away the
					// old one, otherwise throw away the new one
					if((newFaceCollision->where - startPos).magSq()
						< (faceCollision->where - startPos).magSq())
					{
						delete faceCollision;
						faceCollision = newFaceCollision;
					}
					else
						delete newFaceCollision;
				}
				else
				{
					collisionFound = FACE;
					faceCollision = newFaceCollision;
				}
			}
			else // there wasn't a face collision, so check for edge/corner collision
			{
				startPos = theBall.getPosition();
				endPos = startPos + (theBall.getVelocity() * timeStep);
				
				float alphas[6];
				
				alphas[0] = linePointMinimumDistance(startPos, endPos,
									  				       triangles[x]->getV1());
				alphas[1] = linePointMinimumDistance(startPos, endPos,
													       triangles[x]->getV2());
				alphas[2] = linePointMinimumDistance(startPos, endPos,
													       triangles[x]->getV3());

				float betas[3] = { 0.0, 0.0, 0.0 };
				lineLineMinimumDistance(startPos, endPos,
										triangles[x]->getV1(), triangles[x]->getV2(),
										alphas[3], betas[0]);
				lineLineMinimumDistance(startPos, endPos,
										triangles[x]->getV2(), triangles[x]->getV3(),
										alphas[4], betas[1]);
				lineLineMinimumDistance(startPos, endPos,
										triangles[x]->getV3(), triangles[x]->getV1(),
										alphas[5], betas[2]);

				// find which of the collisions that actually happen (i.e. minimum distance
				// is <= ballRadius) happens earliest (i.e. has smallest alpha that's >= 0.0f),
				// and if any of them happen earlier than any already-existing edge or corner
				// collision from a previous triangle (i.e. have a smaller alpha than previousAlpha
				// and shorter distance than faceCollision)

				for(int v = 0; v < 6; v++)
				{
					Vector closestPt = startPos + (endPos-startPos) * alphas[v];//startPos + (endPos - startPos) * alphas[v];

					Vector vertex;
					if(v == 0)
						vertex = triangles[x]->getV1();
					else if(v == 1)
						vertex = triangles[x]->getV2();
					else if(v == 2)
						vertex = triangles[x]->getV3();
					else if(v == 3)
						vertex = triangles[x]->getV1() + (triangles[x]->getV2() - triangles[x]->getV1()) * betas[0];
					else if(v == 4)
						vertex = triangles[x]->getV2() + (triangles[x]->getV3() - triangles[x]->getV2()) * betas[1];
					else
						vertex = triangles[x]->getV3() + (triangles[x]->getV1() - triangles[x]->getV3()) * betas[2];

					// first check if this collision happens at all; if not, skip
					if((closestPt - vertex).magSq() > theBall.getRadius() * theBall.getRadius()
						|| alphas[v] <= 0.00f)
						continue;

					Vector pathCollisionPt;  // point on the path where we collide (i.e. where we're exactly
										     //  radius away from the collision point on the triangle

					// Use the laws of sines and cosines to 
					float distance = sqrt((closestPt - vertex).magSq());
					Vector d = closestPt - vertex;
					Vector vel = theBall.getVelocity();
					float r = theBall.getRadius();
					float theta = d.dot(vel) / sqrt(d.magSq()) / sqrt(vel.magSq());
					theta = 3.14159 - theta;
					float phi = asin(sqrt(d.magSq())*sin(theta)/r);
					phi = 3.14159 - phi - theta;
					float c = sqrt(d.magSq() + r*r + cos(phi));
					alphas[v] -= c / sqrt((endPos - startPos).magSq());
					pathCollisionPt = startPos + (endPos - startPos) * alphas[v];

					if(alphas[v] < 0.0f)
						alphas[v] = 0.0f;

					// does this collision come before any previous edge/corner collisions?
					if(alphas[v] < smallestAlpha)
					{
						// if we have a face collision, check if this one comes first
						if(collisionFound == FACE)
						{
							continue;   // executing the following code can cause problems - I think pathCollisionPt is wrong
							if((pathCollisionPt - startPos).magSq()
								  < (faceCollision->where - startPos).magSq())
								delete faceCollision;
							else
								continue;
						}

						// if we get here, this is the closest collision so far
						smallestAlpha = alphas[v];
						edgeOrCornerCollisionPt = vertex;
						collisionFound = EDGE_OR_CORNER;
					}
				}
			}
		}
	}

	// If there's no collision, update the ball's position with straight-line motion
	if(collisionFound == NONE)
	{
		theBall.setPosition(endPos);
		theBall.advanceRollingRotation(timeStep);
	}
	else // otherwise handle the collision
	{
		// amount of timeStep remaining (will be calculated later, depending on collision type
		float restOfTimeStep;
		
		// new velocity (also calculated later)
		Vector newVel;

		if(collisionFound == FACE)
		{
			// move us to the collision point
			theBall.setPosition(faceCollision->where);

			// To get the new ball velocity:  1. Transform the old velocity into the standard coordinate space, where the triangle
			// is horizontal.  2. Invert the y-component (vertical part) of the velocity.  3.  Transform the new velocity back into
			// real space.
			const Triangle* theTri = faceCollision->triangle;
			newVel = theBall.getVelocity();
			newVel = theTri->forwardTransform(newVel) - theTri->forwardTransform(Vector(0.0, 0.0, 0.0));
			newVel.y = -newVel.y * ELASTICITY;
			newVel = theTri->reverseTransform(newVel) - theTri->reverseTransform(Vector(0.0, 0.0, 0.0));

			// Friction - subtract (not multiply!) a small amount from the velocity
			Vector newVelNorm = newVel;
			newVelNorm.normalize();
			newVelNorm = newVelNorm * FRICTION;
			if( newVel.magSq() > newVelNorm.magSq() )
				newVel = newVel - newVelNorm;
			else
				newVel = Vector(0,0,0); // friction brings us to a rolling stop

			// Ball rotation - update the rolling speed and direction
			Vector normal = theTri->reverseTransform(Vector(0.0, 1.0, 0.0)) - theTri->reverseTransform(Vector(0.0, 0.0, 0.0));
			float xSq = normal.dot(newVel);
			xSq *= xSq;
			float rollSpeed = sqrt(newVel.magSq() - xSq);
			Vector rotVector = newVel.cross(normal);
			if(!rotVector.magSq())  // if there is no vector to rotate around
				rollSpeed = 0;      // don't try to rotate

			restOfTimeStep = timeStep * (1.0 -
 					sqrt((faceCollision->where - startPos).magSq() / (endPos - startPos).magSq()));

			delete faceCollision;

			theBall.advanceRollingRotation(timeStep - restOfTimeStep); // roll the ball up to the collision
			theBall.setRollingRotation(rotVector, rollSpeed);
		}
		else // EDGE_OR_FACE collision
		{
			Vector collisionPt = startPos + (endPos - startPos) * smallestAlpha;
					 			 // point on the path where we collide (i.e. where we're exactly
			                     //  radius away from the collision point on the triangle
			restOfTimeStep = (1.0f - smallestAlpha) * timeStep;
			// move us to the collision
			theBall.setPosition(collisionPt);

			// do some (correct) stuff here, including calculating newVel and restOfTimeStep

			// To calculate new velocity:
			// We have a current velocity vector and can calculate a vector from the
			//  collision point on the current path and the collision point on the
			//  edge/corner.  The angle between these two vectors is the angle we hit the
			//  edge/corner at (e.g. hitting it straight on would mean they're the same vector).
			// New velocity will be this collision-point to collision-point vector flipped about
			//  the current velocity vector.
			Vector curDir = theBall.getVelocity(); //(endPos - collisionPt);
			Vector collisionDir = edgeOrCornerCollisionPt - collisionPt;
			Vector proj = collisionDir * ( collisionDir.dot(curDir) /
									       collisionDir.magSq()); // projection of current velocity vector
													              // onto collision-angle vector
			Vector perp = curDir - proj;
			newVel = curDir + (perp * 2.0f);
			newVel.normalize();
			newVel = newVel * sqrt(theBall.getVelocity().magSq()) * ELASTICITY;

			theBall.setPosition(theBall.getPosition() + newVel * timeStep);
			restOfTimeStep = 0;

			theBall.advanceRollingRotation(timeStep - restOfTimeStep); // roll up to the collision
		}

		theBall.setVelocity(newVel);
		theCamera.lookAt(endPos);

		// Finish up the rest of the timestep
		advanceTime(restOfTimeStep);
	}

	return (theBall.getVelocity().magSq() < .00005);
}

bool World::boundingSphereIntersection(Vector centerA, float radiusA,
									   Vector centerB, float radiusB)
{
	// true if distance between centers is <= sum of radii
	// (for efficiency, actually check if distance^2 <= (sum of radii)^2)
	float radSum = radiusA + radiusB;
	return (centerA - centerB).magSq() <= radSum*radSum;
}

Collision* World::checkFaceCollision(const Vector& pathStart, const Vector& pathEnd,
									 float ballRadius, const Triangle* triangle)
{
	// Transform everything to standard coordinates
	Vector pathStartStd = triangle->forwardTransform(pathStart);
	Vector pathEndStd = triangle->forwardTransform(pathEnd);
	Vector v1Std = triangle->forwardTransform(triangle->getV1());
	Vector v2Std = triangle->forwardTransform(triangle->getV2());
	Vector v3Std = triangle->forwardTransform(triangle->getV3());

	Vector relGrad = pathEndStd - pathStartStd; // relative gradient (3d "slope") of path

	// there's only a possibility of a face collision if we're above the face and moving
	// down towards it...
	if( relGrad.y < 0.0 && pathStartStd.y >= 0.0)
	{
		// find (x,z) coordinates on the ball's path where y = ballRadius
		// if (a,b,c) is the relative gradient and (x0,y0,z0) is a point on the line,
		// in parametric form, x = x0 + at, y = y0 + bt, z = z0 + ct, so solve for
		// t in ballRadius = y0 + bt, then use this to get x and z

		float t = (ballRadius - pathStartStd.y) * (1.0 / relGrad.y);

		// calculate the collision point and the projection of the collision point to the x-z plane
		Vector point(pathStartStd.x + relGrad.x * t, ballRadius, pathStartStd.z + relGrad.z * t);
		Vector pointXZ = point;
		pointXZ.y = 0.0;

		// Check that the original point (not projected down to x-z) is on the path first,
		// and if so, check that projected-down point is within the triangle (i.e., for each
		// side, y-component of cross-product of v(i+1) - v(i) and point - v(i) must be positive).
		// Return a collision if all these conditions hold.
		if((point - pathStartStd).magSq() <= relGrad.magSq()
		   && (v2Std - v1Std).cross(pointXZ - v1Std).y > 0.0
		   && (v3Std - v2Std).cross(pointXZ - v2Std).y > 0.0
		   && (v1Std - v3Std).cross(pointXZ - v3Std).y > 0.0)
		{
			return new Collision(triangle->reverseTransform(point), triangle);
		}
	}

	return NULL;
}


/************************************************************/
/*															*/
/*	this routine finds the closest point q on the line		*/
/*	segment defined by the points "start" and "end"			*/
/*	to the point "v"										*/
/*															*/
/*	it returns alpha where q=start + alpha * (end-start)	*/
/*															*/
/************************************************************/
				
float World::linePointMinimumDistance(const Vector& start, const Vector& end, const Vector& v)
{
	// construct a vector along the line
	Vector v1=end-start;
	float lenSq = v1.magSq();
	// check if the line is actually a point
	if (lenSq == 0)
		// if so the point start=end is desired point and alpha=0
		return 0; 

	// construct a vector from the line to the point
	Vector v2=v-start;
	// q=start + alpha * v1 is the closest point on the line to the point v
	// where alph is defined below
	float alpha = v1.dot(v2)/(lenSq);

	// if q is not on the line segment then an endpoint of the line
	// segment is the desired point
	if (alpha<0)
		alpha=0;
	if (alpha>1)
		alpha=1;
	
	return alpha;
}

//******************************************************************//
//																	//
//	This routine finds to closest points p1 & p2 on two	line		//
//	segments, l1 (start1 to end1) and l2 (start2 to end2)			//
//  It computes alpha and beta in [0,1] such that					//
//		p1 = start1 + alpha * (end1-start1)							//
//		p2 = start2 + beta * (end2-start2)							//
//	If the lines are parallel -- which should never result in a		//
//	collision, it returns alpha=beta=-1								//
//																	//
//******************************************************************//	

void World::lineLineMinimumDistance(const Vector& start1, const Vector& end1,
									const Vector& start2, const Vector& end2,
									float& alpha, float& beta)
{
	// find vector along first line
	Vector v1=end1-start1;
	float v1dotv1=v1.dot(v1);
	// check if the line is actually a point
	if (v1dotv1==0) {	
		// first line is really a point
		alpha = 0;
		beta = linePointMinimumDistance(start2, end2, start1);
		return;
	}

	// find vector along second line
	Vector v2=end2-start2;
	float v2dotv2=v2.dot(v2);
	// check if the line is actually a point
	if (v2dotv2==0) {
		// second line is really a point
		alpha = linePointMinimumDistance(start1, end1, start2);
		beta=0;
		return;
	}

	// check if lines are parallel
	float v1dotv2=v2.dot(v1);  // we define both for convenience
	float v2dotv1=v1dotv2;

	if (v2dotv2*v1dotv1 != v2dotv1*v1dotv2) {  // do not parallel case first
		// Find closest point
		Vector s=start2-start1;
		float sdotv1=s.dot(v1);
		float sdotv2=s.dot(v2);

		beta = (sdotv2- sdotv1*v1dotv2/v1dotv1)/(v1dotv2*v2dotv1/v1dotv1 - v2dotv2);
		alpha = (sdotv1 + beta*v2dotv1)/(v1dotv1);

		// if alpha and beta are both on their line segments
		// we are done
		// if not, then at least one of the points we are
		// looking for is an endpoint.  we could test each
		// endpoint to find its closest point on the other
		// line.  this would take four calls to linePointMinimumDistance.
		// if we examine the possible cases, however, we
		// need only compute one call to linePointMinimumDistance.
		// we take that approach.

		// alpha and beta are each in one of three states:
		// 0<= alpha <=1, alpha<0, or alpha>1  
		// 0<= beta <=1,  beta <0, or beta>1
		// thus there are nine states to consider
		// testval represents the states
		int testval = 3*(alpha<0) + 6*(alpha>1) + 1*(beta<0) + 2*(beta>1);

		switch (testval) {
		case 0: {	// alpha and beta are in [0,1]
			return;
				}

		case 1: {	// alpha in [0,1], beta<0
				alpha = linePointMinimumDistance(start1,end1,start2);
				beta=0;
				return;
				}

		case 2: {	// alpha in [0,1], beta>1
				alpha = linePointMinimumDistance(start1,end1,end2);
				beta=1;
				return;
				}

		case 3: {	// alpha < 0, beta in [0,1]
				alpha=0;
				beta=linePointMinimumDistance(start2,end2,start1);
				return;
				}

		case 6: {	// alpha > 1, beta in [0,1]
				alpha=1;
				beta=linePointMinimumDistance(start2,end2,end1);
				return;
				}

		case 5: {  // alpha <0, beta >1
				alpha=0;
				beta=1;
				return;
				}

		case 7: {  // alpha >1, beta <0
				alpha=1;
				beta=0;
				return;
				}

		case 4: {  //alpha < 0, beta < 0
					if ((v1*alpha).magSq() > (v2*beta).magSq()) {
						alpha=0;
						beta=linePointMinimumDistance(start2,end2,start1);
					}
					else {
						beta=0;
						alpha = linePointMinimumDistance(start1,end1,start2);
					}
					return;
				}

		case 8: {  // alpha > 1, beta > 1
					if ((start1+v1*alpha-end1).magSq() > (start2+v2*beta).magSq()) {
						alpha=1;
						beta=linePointMinimumDistance(start2,end2,end1);
					}
					else {
						beta = 1;
						alpha =	linePointMinimumDistance(start1,end1,end2);
					}

					return;
				}
		}
	}
	else { 

		// the lines are parallel
		// this should never result in a collision
		alpha = -1;
		beta=-1;
		return;
	}
}

void World::mouseMove(int newx, int newy)
{
	if(mode == PULLBACK)
	{
		theCamera.setRelativePosition(0, newx - mousex, newy - mousey);
		mousex = newx;
		mousey = newy;
	} else {
		theCamera.setRelativePosition(0, newx - mousex, newy - mousey);
		theClub.setTheta(-theCamera.getTheta());
		mousex = newx;
		mousey = newy;
	}
	draw();
}

void World::mousePress(int button, int state, int x, int y)
{
	mousex = x;
	mousey = y;
	if(button == GLUT_RIGHT_BUTTON && mode == AIM)
	{
		mode = PULLBACK;
		glutTimerFunc(0, hitTimerCallback, 0);
		return;
	}
	if(button == GLUT_RIGHT_BUTTON && mode == PULLBACK)
	{
		mode = HIT;  // notify timer function that we're done pulling back club
		power = theClub.getSwingAngle();
	}
}

void World::hitTimer()
{
	float angle = theClub.getSwingAngle();
	glutPostRedisplay();

	if(angle > 60)
	{
		power = 60;
		mode = HIT;
	}

	if(mode == HIT)
	{
		theClub.setSwingAngle(angle - 3);
		if(angle <= 0)
		{
			mode = SIM;
			theClub.setSwingAngle(0);
			float xv = (float) -sin((double) 3.14159/180.0*theClub.getTheta());
			float zv = (float) -cos((double) 3.14159/180.0*theClub.getTheta());
			theBall.setVelocity(Vector(xv, 0, zv)*power*.01);
			numStrokes++;
			glutTimerFunc(0, physTimerCallback, 0);
			return;
		} else {
			glutTimerFunc(10, hitTimerCallback, 0);
			return;
		}
	}
	if(mode == PULLBACK)
		theClub.setSwingAngle(angle + 1);
	glutTimerFunc(10, hitTimerCallback, 0);
}

void World::physTimer()
{
	if(World::instance()->advanceTimestep())
		numSlowSteps++;
	else
		numSlowSteps = 0;
	
	glutPostRedisplay();

	if(mode != SIM)
		return;

	if(numSlowSteps > 50)
	{
		numSlowSteps = 0;
		theClub.aimAt(theBall.getPosition());
		mode = AIM;
		return;
	} else {
		glutTimerFunc(10, physTimerCallback, 0);
	}
}

void World::keyPress(unsigned char key, int x, int y)
{
	if(key=='q')
		exit(0);
	if(key == 'r')
	{
		mode = AIM;
		theCamera.lookAt(TEE_POSITION);
		theClub.aimAt(TEE_POSITION);
		theBall.setPosition(TEE_POSITION);
	}

	if(key == 'z')
		theCamera.setRelativePosition(0.1, 0, 0);
	if(key == 'x')
		theCamera.setRelativePosition(-0.1, 0, 0);
	glutPostRedisplay();
}