// The world.  Implemented as a singleton class.

#pragma once

#include "Ball.h"
#include "Triangle.h"
#include "includes.h"
#include "camera.h"
#include "club.h"
#include "Hole.h"

enum GameMode { SIM, AIM, PULLBACK, HIT, DONE }; // SIM - physics simulation.  
                                                 // AIM - aiming the club.  
										         // PULLBACK - pulling club back
										         // HIT - hitting the ball
												 // DONE - ball went in the hole

enum CollisionType { NONE, FACE, EDGE_OR_CORNER };

class World
{
public:
	~World(void);

	// constants
	static const float BALL_RADIUS;
	static const float ELASTICITY;
	static const float GRAVITY;
	static const float FRICTION;
	static const Vector HOLE_POSITION;
	static const float HOLE_RADIUS;
	static const Vector TEE_POSITION;

	// return a pointer to the singleton instance of the World
	static World* instance();

	// Loads in world polygons from the file fileName, returns true if successful, false otherwise
	bool loadFile(char* fileName);

	// Redraw everything in the world
	void draw();

	// Advance a timestep. Return true if at least one collision occured in this time,
	// false otherwise (for testing purposes only; normally nobody else really needs to know
	// if collisions happened or not, since World handles them itself)
	bool advanceTimestep();

	// Set ball velocity to a new value
	// (this is only for the testing harness; will probably remove in final version, since
	//  except for testing purposes, the world should be handling all the velocity updating)
	void setBallVelocity(Vector newVelocity) { theBall.setVelocity(newVelocity); }

	// mouse movement callback (only called while a mouse button is held down)
	void mouseMove(int newx, int newy);
	// Mouse button press callback
	void mousePress(int button, int state, int x, int y);
	// Keyboard button callback
	void keyPress(unsigned char key, int x, int y);
	// callback for pulling the club back when hitting
	void hitTimer();
	// callback for physics simulation
	void physTimer();
	// the above functions can't be called directly by glut because of C++ internal stuff
	// glut can call static functions so we make some here that just call the above functions
	static void mouseMoveCallback(int newx, int newy) {instance()->mouseMove(newx, newy);};
	static void mousePressCallback(int button, int state, int x, int y) {instance()->mousePress(button, state, x, y);};
	static void keyPressCallback(unsigned char key, int x, int y) {instance()->keyPress(key,x,y); };
	static void hitTimerCallback(int dummy) { instance()->hitTimer(); };
	static void physTimerCallback(int dummy) { instance()->physTimer(); };

private:
	// Private constructor to enforce singleton constraint
	World(void);

	static World* pinstance;

	Ball theBall;
	Camera theCamera;
	Club theClub;
	Hole theHole;
	GameMode mode;

	Triangle** triangles;
	int numTriangles;
	int numSlowSteps;
	int numStrokes;
	float power;

	int mousex, mousey;

	// Advance a certain time, where 1.0 is one timestep.  This allows us to finish partial
	// timesteps after bounces (by using time < 1.0).
	bool advanceTime(float timeStep);

	// Quick check to see if two bounding spheres intersect.
	static bool boundingSphereIntersection(Vector centerA, float radiusA,
										   Vector centerB, float radiusB);

	// Check collision with the face of a triangle, returning either the point of the
	// collision (in a newly allocated Vector the caller must delete) or NULL if no collision
	// Note: only checks for collisions with front faces!
	static Collision* checkFaceCollision(const Vector& pathStart, const Vector& pathEnd,
										 float ballRadius, const Triangle* triangle);

	// Next two routines are used for detecting collisions with points and lines, respectively,
	// and were mostly written by Prof. Z.  See comments in the .cpp file for details.
	static float linePointMinimumDistance(const Vector& start, const Vector& end, const Vector& v);
	static void lineLineMinimumDistance(const Vector& start1, const Vector& end1,
										const Vector& start2, const Vector& end2,
										float& alpha, float& beta);
};
